@@ -16,6 +16,8 @@ import static java.util.Collections.singletonList;
 import static org.eclipse.che.api.workspace.shared.Constants.WORKSPACE_INFRASTRUCTURE_NAMESPACE_ATTRIBUTE;
 import static org.eclipse.che.workspace.infrastructure.kubernetes.api.shared.KubernetesNamespaceMeta.DEFAULT_ATTRIBUTE;
 import static org.eclipse.che.workspace.infrastructure.kubernetes.api.shared.KubernetesNamespaceMeta.PHASE_ATTRIBUTE;
+import static org.eclipse.che.workspace.infrastructure.kubernetes.namespace.AbstractWorkspaceServiceAccount.CREDENTIALS_SECRET_NAME;
+import static org.eclipse.che.workspace.infrastructure.kubernetes.namespace.AbstractWorkspaceServiceAccount.SECRETS_ROLE_NAME;
 import static org.eclipse.che.workspace.infrastructure.kubernetes.namespace.KubernetesNamespaceFactory.NAMESPACE_TEMPLATE_ATTRIBUTE;
 import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.ArgumentMatchers.anyMap;
@@ -33,6 +35,7 @@ import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertFalse;
 import static org.testng.Assert.assertNull;
 import static org.testng.Assert.assertTrue;
+import static org.testng.Assert.fail;
 
 import ch.qos.logback.classic.spi.LoggingEvent;
 import ch.qos.logback.core.Appender;
@@ -40,15 +43,19 @@ import com.google.common.collect.ImmutableMap;
 import io.fabric8.kubernetes.api.model.Namespace;
 import io.fabric8.kubernetes.api.model.NamespaceBuilder;
 import io.fabric8.kubernetes.api.model.NamespaceList;
+import io.fabric8.kubernetes.api.model.ObjectMeta;
+import io.fabric8.kubernetes.api.model.Secret;
 import io.fabric8.kubernetes.api.model.ServiceAccountList;
 import io.fabric8.kubernetes.api.model.Status;
 import io.fabric8.kubernetes.api.model.rbac.ClusterRoleBuilder;
+import io.fabric8.kubernetes.api.model.rbac.PolicyRule;
 import io.fabric8.kubernetes.api.model.rbac.Role;
 import io.fabric8.kubernetes.api.model.rbac.RoleBindingList;
 import io.fabric8.kubernetes.api.model.rbac.RoleList;
 import io.fabric8.kubernetes.client.KubernetesClient;
 import io.fabric8.kubernetes.client.KubernetesClientException;
 import io.fabric8.kubernetes.client.dsl.FilterWatchListDeletable;
+import io.fabric8.kubernetes.client.dsl.MixedOperation;
 import io.fabric8.kubernetes.client.dsl.NonNamespaceOperation;
 import io.fabric8.kubernetes.client.dsl.Resource;
 import io.fabric8.kubernetes.client.server.mock.KubernetesServer;
@@ -57,6 +64,7 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Set;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
@@ -76,6 +84,7 @@ import org.eclipse.che.commons.subject.SubjectImpl;
 import org.eclipse.che.inject.ConfigurationException;
 import org.eclipse.che.workspace.infrastructure.kubernetes.CheServerKubernetesClientFactory;
 import org.eclipse.che.workspace.infrastructure.kubernetes.KubernetesClientFactory;
+import org.eclipse.che.workspace.infrastructure.kubernetes.api.server.impls.KubernetesNamespaceMetaImpl;
 import org.eclipse.che.workspace.infrastructure.kubernetes.api.shared.KubernetesNamespaceMeta;
 import org.eclipse.che.workspace.infrastructure.kubernetes.util.KubernetesSharedPool;
 import org.mockito.ArgumentCaptor;
@@ -115,8 +124,7 @@ public class KubernetesNamespaceFactoryTest {
   @Mock private PreferenceManager preferenceManager;
   @Mock Appender mockedAppender;
 
-  @Mock
-  private NonNamespaceOperation<Namespace, NamespaceList, Resource<Namespace>> namespaceOperation;
+  @Mock private NonNamespaceOperation namespaceOperation;
 
   @Mock private Resource<Namespace> namespaceResource;
 
@@ -165,6 +173,7 @@ public class KubernetesNamespaceFactoryTest {
             "<username>-che",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -192,6 +201,7 @@ public class KubernetesNamespaceFactoryTest {
             "<username>-che",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -203,6 +213,27 @@ public class KubernetesNamespaceFactoryTest {
     namespaceFactory.checkIfNamespaceIsAllowed("any-namespace");
   }
 
+  @Test
+  public void shouldNormaliseNamespaceWhenUserNameStartsWithKube() {
+    namespaceFactory =
+        new KubernetesNamespaceFactory(
+            "",
+            "",
+            "che-<userid>",
+            true,
+            true,
+            true,
+            NAMESPACE_LABELS,
+            NAMESPACE_ANNOTATIONS,
+            clientFactory,
+            cheClientFactory,
+            userManager,
+            preferenceManager,
+            pool);
+
+    assertEquals("che-kube-admin", namespaceFactory.normalizeNamespaceName("kube:admin"));
+  }
+
   @Test(
       expectedExceptions = ValidationException.class,
       expectedExceptionsMessageRegExp =
@@ -217,6 +248,7 @@ public class KubernetesNamespaceFactoryTest {
             "<username>-che",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -239,6 +271,7 @@ public class KubernetesNamespaceFactoryTest {
             null,
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -289,6 +322,7 @@ public class KubernetesNamespaceFactoryTest {
             "<username>-che",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -332,6 +366,7 @@ public class KubernetesNamespaceFactoryTest {
             "<username>-che",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -362,6 +397,7 @@ public class KubernetesNamespaceFactoryTest {
             "<username>-che",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -395,6 +431,7 @@ public class KubernetesNamespaceFactoryTest {
             "<username>-che",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -423,6 +460,7 @@ public class KubernetesNamespaceFactoryTest {
             "<username>-che",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -442,6 +480,82 @@ public class KubernetesNamespaceFactoryTest {
             .get(PHASE_ATTRIBUTE)); // no phase - means such namespace does not exist
   }
 
+  @Test
+  public void shouldCreateCredentialsSecretIfNotExists() throws Exception {
+    // given
+    namespaceFactory =
+        spy(
+            new KubernetesNamespaceFactory(
+                "",
+                "",
+                "<username>-che",
+                true,
+                true,
+                true,
+                NAMESPACE_LABELS,
+                NAMESPACE_ANNOTATIONS,
+                clientFactory,
+                cheClientFactory,
+                userManager,
+                preferenceManager,
+                pool));
+    KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
+    KubernetesSecrets secrets = mock(KubernetesSecrets.class);
+    when(toReturnNamespace.secrets()).thenReturn(secrets);
+    when(secrets.get()).thenReturn(Collections.emptyList());
+    doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
+    MixedOperation mixedOperation = mock(MixedOperation.class);
+    lenient().when(k8sClient.secrets()).thenReturn(mixedOperation);
+    lenient().when(mixedOperation.inNamespace(anyString())).thenReturn(namespaceOperation);
+
+    // when
+    RuntimeIdentity identity =
+        new RuntimeIdentityImpl("workspace123", null, USER_ID, "workspace123");
+    namespaceFactory.getOrCreate(identity);
+
+    // then
+    ArgumentCaptor<Secret> secretsCaptor = ArgumentCaptor.forClass(Secret.class);
+    verify(namespaceOperation).create(secretsCaptor.capture());
+    Secret secret = secretsCaptor.getValue();
+    Assert.assertEquals(secret.getMetadata().getName(), CREDENTIALS_SECRET_NAME);
+    Assert.assertEquals(secret.getType(), "opaque");
+  }
+
+  @Test
+  public void shouldNotCreateCredentialsSecretIfExists() throws Exception {
+    // given
+    namespaceFactory =
+        spy(
+            new KubernetesNamespaceFactory(
+                "",
+                "",
+                "<username>-che",
+                true,
+                true,
+                true,
+                NAMESPACE_LABELS,
+                NAMESPACE_ANNOTATIONS,
+                clientFactory,
+                cheClientFactory,
+                userManager,
+                preferenceManager,
+                pool));
+    KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
+    prepareNamespace(toReturnNamespace);
+    doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
+    MixedOperation mixedOperation = mock(MixedOperation.class);
+    lenient().when(k8sClient.secrets()).thenReturn(mixedOperation);
+    lenient().when(mixedOperation.inNamespace(anyString())).thenReturn(namespaceOperation);
+
+    // when
+    RuntimeIdentity identity =
+        new RuntimeIdentityImpl("workspace123", null, USER_ID, "workspace123");
+    namespaceFactory.getOrCreate(identity);
+
+    // then
+    verify(namespaceOperation, never()).create(any());
+  }
+
   @Test(
       expectedExceptions = InfrastructureException.class,
       expectedExceptionsMessageRegExp =
@@ -454,6 +568,7 @@ public class KubernetesNamespaceFactoryTest {
             "<username>-che",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -479,6 +594,7 @@ public class KubernetesNamespaceFactoryTest {
             "<username>-che",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -509,6 +625,7 @@ public class KubernetesNamespaceFactoryTest {
                 "<username>-che",
                 true,
                 true,
+                true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
                 clientFactory,
@@ -517,6 +634,7 @@ public class KubernetesNamespaceFactoryTest {
                 preferenceManager,
                 pool));
     KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
+    prepareNamespace(toReturnNamespace);
     doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
 
     // when
@@ -526,7 +644,7 @@ public class KubernetesNamespaceFactoryTest {
     // then
     assertEquals(toReturnNamespace, namespace);
     verify(namespaceFactory, never()).doCreateServiceAccount(any(), any());
-    verify(toReturnNamespace).prepare(eq(false), any());
+    verify(toReturnNamespace).prepare(eq(false), any(), any());
   }
 
   @Test
@@ -540,6 +658,7 @@ public class KubernetesNamespaceFactoryTest {
                 "<username>-che",
                 false,
                 true,
+                true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
                 clientFactory,
@@ -548,6 +667,7 @@ public class KubernetesNamespaceFactoryTest {
                 preferenceManager,
                 pool));
     KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
+    prepareNamespace(toReturnNamespace);
     doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
 
     // when
@@ -558,7 +678,7 @@ public class KubernetesNamespaceFactoryTest {
     // then
     assertEquals(toReturnNamespace, namespace);
     verify(namespaceFactory, never()).doCreateServiceAccount(any(), any());
-    verify(toReturnNamespace).prepare(eq(false), any());
+    verify(toReturnNamespace).prepare(eq(false), any(), any());
   }
 
   @Test
@@ -573,6 +693,7 @@ public class KubernetesNamespaceFactoryTest {
                 "<username>-che",
                 true,
                 true,
+                true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
                 clientFactory,
@@ -581,6 +702,7 @@ public class KubernetesNamespaceFactoryTest {
                 preferenceManager,
                 pool));
     KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
+    prepareNamespace(toReturnNamespace);
     when(toReturnNamespace.getWorkspaceId()).thenReturn("workspace123");
     when(toReturnNamespace.getName()).thenReturn("workspace123");
     doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
@@ -610,6 +732,7 @@ public class KubernetesNamespaceFactoryTest {
                 "<username>-che",
                 true,
                 true,
+                true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
                 clientFactory,
@@ -618,6 +741,7 @@ public class KubernetesNamespaceFactoryTest {
                 preferenceManager,
                 pool));
     KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
+    prepareNamespace(toReturnNamespace);
     when(toReturnNamespace.getWorkspaceId()).thenReturn("workspace123");
     when(toReturnNamespace.getName()).thenReturn("workspace123");
     doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
@@ -653,7 +777,7 @@ public class KubernetesNamespaceFactoryTest {
     RoleList roles = k8sClient.rbac().roles().inNamespace("workspace123").list();
     assertEquals(
         roles.getItems().stream().map(r -> r.getMetadata().getName()).collect(Collectors.toSet()),
-        Sets.newHashSet("workspace-view", "workspace-metrics", "exec"));
+        Sets.newHashSet("workspace-view", "workspace-metrics", "workspace-secrets", "exec"));
     RoleBindingList bindings = k8sClient.rbac().roleBindings().inNamespace("workspace123").list();
     assertEquals(
         bindings
@@ -666,7 +790,67 @@ public class KubernetesNamespaceFactoryTest {
             "serviceAccount-cluster0",
             "serviceAccount-cluster1",
             "serviceAccount-view",
-            "serviceAccount-exec"));
+            "serviceAccount-exec",
+            "serviceAccount-secrets"));
+  }
+
+  @Test
+  public void shouldCreateAndBindCredentialsSecretRole() throws Exception {
+    // given
+    namespaceFactory =
+        spy(
+            new KubernetesNamespaceFactory(
+                "serviceAccount",
+                "cr2, cr3",
+                "<username>-che",
+                true,
+                true,
+                true,
+                NAMESPACE_LABELS,
+                NAMESPACE_ANNOTATIONS,
+                clientFactory,
+                cheClientFactory,
+                userManager,
+                preferenceManager,
+                pool));
+    KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
+    prepareNamespace(toReturnNamespace);
+    when(toReturnNamespace.getWorkspaceId()).thenReturn("workspace123");
+    when(toReturnNamespace.getName()).thenReturn("workspace123");
+    doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
+    when(clientFactory.create(any())).thenReturn(k8sClient);
+
+    // when
+    RuntimeIdentity identity =
+        new RuntimeIdentityImpl("workspace123", null, USER_ID, "workspace123");
+    namespaceFactory.getOrCreate(identity);
+
+    // then
+    Optional<Role> roleOptional =
+        k8sClient
+            .rbac()
+            .roles()
+            .inNamespace("workspace123")
+            .list()
+            .getItems()
+            .stream()
+            .filter(r -> r.getMetadata().getName().equals(SECRETS_ROLE_NAME))
+            .findAny();
+    assertTrue(roleOptional.isPresent());
+    PolicyRule rule = roleOptional.get().getRules().get(0);
+    assertEquals(rule.getResources(), singletonList("secrets"));
+    assertEquals(rule.getResourceNames(), singletonList(CREDENTIALS_SECRET_NAME));
+    assertEquals(rule.getApiGroups(), singletonList(""));
+    assertEquals(rule.getVerbs(), Arrays.asList("get", "patch"));
+    assertTrue(
+        k8sClient
+            .rbac()
+            .roleBindings()
+            .inNamespace("workspace123")
+            .list()
+            .getItems()
+            .stream()
+            .anyMatch(rb -> rb.getMetadata().getName().equals("serviceAccount-secrets")));
   }
 
   @Test
@@ -680,6 +864,7 @@ public class KubernetesNamespaceFactoryTest {
                 "<username>-che",
                 true,
                 true,
+                true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
                 clientFactory,
@@ -688,6 +873,7 @@ public class KubernetesNamespaceFactoryTest {
                 preferenceManager,
                 pool));
     KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
+    prepareNamespace(toReturnNamespace);
     when(toReturnNamespace.getWorkspaceId()).thenReturn("workspace123");
     when(toReturnNamespace.getName()).thenReturn("workspace123");
     doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
@@ -709,7 +895,7 @@ public class KubernetesNamespaceFactoryTest {
     RoleList roles = k8sClient.rbac().roles().inNamespace("workspace123").list();
     assertEquals(
         roles.getItems().stream().map(r -> r.getMetadata().getName()).collect(Collectors.toSet()),
-        Sets.newHashSet("workspace-view", "workspace-metrics", "exec"));
+        Sets.newHashSet("workspace-view", "workspace-metrics", "workspace-secrets", "exec"));
     Role role1 = roles.getItems().get(0);
     Role role2 = roles.getItems().get(1);
 
@@ -725,7 +911,11 @@ public class KubernetesNamespaceFactoryTest {
             .stream()
             .map(r -> r.getMetadata().getName())
             .collect(Collectors.toSet()),
-        Sets.newHashSet("serviceAccount-metrics", "serviceAccount-view", "serviceAccount-exec"));
+        Sets.newHashSet(
+            "serviceAccount-metrics",
+            "serviceAccount-view",
+            "serviceAccount-exec",
+            "serviceAccount-secrets"));
   }
 
   @Test
@@ -737,6 +927,7 @@ public class KubernetesNamespaceFactoryTest {
             "che-<userid>",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -756,6 +947,7 @@ public class KubernetesNamespaceFactoryTest {
             "che-<userid>",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -779,6 +971,7 @@ public class KubernetesNamespaceFactoryTest {
             "che-<userid>",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -804,6 +997,7 @@ public class KubernetesNamespaceFactoryTest {
             "che-<userid>",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -834,6 +1028,7 @@ public class KubernetesNamespaceFactoryTest {
             "che-<userid>-<username>",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -865,6 +1060,7 @@ public class KubernetesNamespaceFactoryTest {
             "che-<userid>-<username>",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -886,6 +1082,34 @@ public class KubernetesNamespaceFactoryTest {
     assertEquals(namespace, "che-user123-jondoe");
   }
 
+  @Test
+  public void testEvalNamespaceKubeAdmin() throws Exception {
+    namespaceFactory =
+        spy(
+            new KubernetesNamespaceFactory(
+                "",
+                "",
+                "che-<username>",
+                true,
+                true,
+                true,
+                NAMESPACE_LABELS,
+                NAMESPACE_ANNOTATIONS,
+                clientFactory,
+                cheClientFactory,
+                userManager,
+                preferenceManager,
+                pool));
+    doReturn(Optional.empty()).when(namespaceFactory).fetchNamespace(anyString());
+
+    String namespace =
+        namespaceFactory.evaluateNamespaceName(
+            new NamespaceResolutionContext("workspace123", "kube:admin", "kube:admin"));
+
+    assertTrue(namespace.startsWith("che-kube-admin-"));
+    assertEquals(namespace.length(), 21);
+  }
+
   @Test
   public void testEvalNamespaceUsesWorkspaceRecordedNamespaceIfWorkspaceRecordsIt()
       throws Exception {
@@ -896,6 +1120,7 @@ public class KubernetesNamespaceFactoryTest {
             "che-<userid>",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -925,6 +1150,7 @@ public class KubernetesNamespaceFactoryTest {
             "che-<userid>",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -975,6 +1201,7 @@ public class KubernetesNamespaceFactoryTest {
             "<username>-che",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -990,6 +1217,124 @@ public class KubernetesNamespaceFactoryTest {
     assertEquals(namespace, "ns1");
   }
 
+  @Test
+  public void shouldHandleProvision() throws InfrastructureException {
+    // given
+    namespaceFactory =
+        spy(
+            new KubernetesNamespaceFactory(
+                "",
+                "",
+                "<username>-che",
+                false,
+                true,
+                true,
+                NAMESPACE_LABELS,
+                NAMESPACE_ANNOTATIONS,
+                clientFactory,
+                cheClientFactory,
+                userManager,
+                preferenceManager,
+                pool));
+    KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
+    prepareNamespace(toReturnNamespace);
+    when(toReturnNamespace.getName()).thenReturn("jondoe-che");
+    doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
+    KubernetesNamespaceMetaImpl namespaceMeta =
+        new KubernetesNamespaceMetaImpl(
+            "jondoe-che", ImmutableMap.of("phase", "active", "default", "true"));
+    doReturn(Optional.of(namespaceMeta)).when(namespaceFactory).fetchNamespace(eq("jondoe-che"));
+
+    // when
+    NamespaceResolutionContext context =
+        new NamespaceResolutionContext("workspace123", "user123", "jondoe");
+    KubernetesNamespaceMeta actual = namespaceFactory.provision(context);
+
+    // then
+    assertEquals(actual.getName(), "jondoe-che");
+    assertEquals(actual.getAttributes(), ImmutableMap.of("phase", "active", "default", "true"));
+  }
+
+  @Test(
+      expectedExceptions = InfrastructureException.class,
+      expectedExceptionsMessageRegExp = "Not able to find namespace jondoe-cha-cha-cha")
+  public void shouldFailToProvisionIfNotAbleToFindNamespace() throws InfrastructureException {
+    // given
+    namespaceFactory =
+        spy(
+            new KubernetesNamespaceFactory(
+                "",
+                "",
+                "<username>-cha-cha-cha",
+                false,
+                true,
+                true,
+                NAMESPACE_LABELS,
+                NAMESPACE_ANNOTATIONS,
+                clientFactory,
+                cheClientFactory,
+                userManager,
+                preferenceManager,
+                pool));
+    KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
+    prepareNamespace(toReturnNamespace);
+    when(toReturnNamespace.getName()).thenReturn("jondoe-cha-cha-cha");
+    doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
+    KubernetesNamespaceMetaImpl namespaceMeta =
+        new KubernetesNamespaceMetaImpl(
+            "jondoe-cha-cha-cha", ImmutableMap.of("phase", "active", "default", "true"));
+    doReturn(Optional.empty()).when(namespaceFactory).fetchNamespace(eq("jondoe-cha-cha-cha"));
+
+    // when
+    NamespaceResolutionContext context =
+        new NamespaceResolutionContext("workspace123", "user123", "jondoe");
+    namespaceFactory.provision(context);
+
+    // then
+    fail("should not reach this point since exception has to be thrown");
+  }
+
+  @Test(
+      expectedExceptions = InfrastructureException.class,
+      expectedExceptionsMessageRegExp = "Error occurred when tried to fetch default namespace")
+  public void shouldFail2ProvisionIfNotAbleToFindNamespace() throws InfrastructureException {
+    // given
+    namespaceFactory =
+        spy(
+            new KubernetesNamespaceFactory(
+                "",
+                "",
+                "<username>-cha-cha-cha",
+                false,
+                true,
+                true,
+                NAMESPACE_LABELS,
+                NAMESPACE_ANNOTATIONS,
+                clientFactory,
+                cheClientFactory,
+                userManager,
+                preferenceManager,
+                pool));
+    KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
+    prepareNamespace(toReturnNamespace);
+    when(toReturnNamespace.getName()).thenReturn("jondoe-cha-cha-cha");
+    doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
+    KubernetesNamespaceMetaImpl namespaceMeta =
+        new KubernetesNamespaceMetaImpl(
+            "jondoe-cha-cha-cha", ImmutableMap.of("phase", "active", "default", "true"));
+    doThrow(new InfrastructureException("Error occurred when tried to fetch default namespace"))
+        .when(namespaceFactory)
+        .fetchNamespace(eq("jondoe-cha-cha-cha"));
+
+    // when
+    NamespaceResolutionContext context =
+        new NamespaceResolutionContext("workspace123", "user123", "jondoe");
+    namespaceFactory.provision(context);
+
+    // then
+    fail("should not reach this point since exception has to be thrown");
+  }
+
   @Test
   public void testUsernamePlaceholderInLabelsIsNotEvaluated() throws InfrastructureException {
     List<Namespace> namespaces =
@@ -1012,6 +1357,7 @@ public class KubernetesNamespaceFactoryTest {
             "<username>-che",
             true,
             true,
+            true,
             "try_placeholder_here=<username>",
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -1025,6 +1371,41 @@ public class KubernetesNamespaceFactoryTest {
     verify(namespaceOperation).withLabels(Map.of("try_placeholder_here", "<username>"));
   }
 
+  @Test
+  public void testUsernamePlaceholderInAnnotationsIsEvaluated() throws InfrastructureException {
+
+    // given
+    namespaceFactory =
+        spy(
+            new KubernetesNamespaceFactory(
+                "",
+                "",
+                "<username>-che",
+                true,
+                true,
+                true,
+                NAMESPACE_LABELS,
+                "try_placeholder_here=<username>",
+                clientFactory,
+                cheClientFactory,
+                userManager,
+                preferenceManager,
+                pool));
+    EnvironmentContext.getCurrent().setSubject(new SubjectImpl("jondoe", "123", null, false));
+    KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
+    prepareNamespace(toReturnNamespace);
+    doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
+
+    // when
+    RuntimeIdentity identity = new RuntimeIdentityImpl("workspace123", null, USER_ID, "old-che");
+    KubernetesNamespace namespace = namespaceFactory.getOrCreate(identity);
+
+    // then
+    assertEquals(toReturnNamespace, namespace);
+    verify(toReturnNamespace)
+        .prepare(eq(false), any(), eq(Map.of("try_placeholder_here", "jondoe")));
+  }
+
   @Test(dataProvider = "invalidUsernames")
   public void normalizeTest(String raw, String expected) {
     namespaceFactory =
@@ -1034,6 +1415,7 @@ public class KubernetesNamespaceFactoryTest {
             "<username>-che",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -1053,6 +1435,7 @@ public class KubernetesNamespaceFactoryTest {
             "che-<userid>",
             true,
             true,
+            true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
             clientFactory,
@@ -1132,6 +1515,16 @@ public class KubernetesNamespaceFactoryTest {
     when(namespaceList.getItems()).thenThrow(e);
   }
 
+  private void prepareNamespace(KubernetesNamespace namespace) throws InfrastructureException {
+    KubernetesSecrets secrets = mock(KubernetesSecrets.class);
+    when(namespace.secrets()).thenReturn(secrets);
+    Secret secretMock = mock(Secret.class);
+    ObjectMeta objectMeta = mock(ObjectMeta.class);
+    when(objectMeta.getName()).thenReturn(CREDENTIALS_SECRET_NAME);
+    when(secretMock.getMetadata()).thenReturn(objectMeta);
+    when(secrets.get()).thenReturn(Collections.singletonList(secretMock));
+  }
+
   private Namespace createNamespace(String name, String phase) {
     return new NamespaceBuilder()
         .withNewMetadata()
