@@ -68,7 +68,8 @@ public class AuthorizingFileContentProvider<T extends RemoteFactoryUrl>
               EnvironmentContext.getCurrent().getSubject(), remoteFactoryUrl.getHostName());
       if (token.isPresent()) {
         PersonalAccessToken personalAccessToken = token.get();
-        String content = urlFetcher.fetch(requestURL, "Bearer " + personalAccessToken.getToken());
+        String content =
+            urlFetcher.fetch(requestURL, formatAuthorization(personalAccessToken.getToken()));
         gitCredentialManager.createOrReplace(personalAccessToken);
         return content;
       } else {
@@ -81,27 +82,26 @@ public class AuthorizingFileContentProvider<T extends RemoteFactoryUrl>
                 personalAccessTokenManager.fetchAndSave(
                     EnvironmentContext.getCurrent().getSubject(), remoteFactoryUrl.getHostName());
             String content =
-                urlFetcher.fetch(requestURL, "Bearer " + personalAccessToken.getToken());
+                urlFetcher.fetch(requestURL, formatAuthorization(personalAccessToken.getToken()));
             gitCredentialManager.createOrReplace(personalAccessToken);
             return content;
-          } catch (ScmUnauthorizedException
-              | ScmCommunicationException
-              | UnknownScmProviderException e) {
+          } catch (ScmUnauthorizedException | UnknownScmProviderException e) {
             throw new DevfileException(e.getMessage(), e);
+          } catch (ScmCommunicationException e) {
+            throw new IOException(
+                String.format(
+                    "Failed to fetch a content from URL %s. Make sure the URL"
+                        + " is correct. For private repository, make sure authentication is configured."
+                        + " Additionally, if you're using "
+                        + " relative form, make sure the referenced file are actually stored"
+                        + " relative to the devfile on the same host,"
+                        + " or try to specify URL in absolute form. The current attempt to authenticate"
+                        + " request, failed with the following error message: %s",
+                    fileURL, e.getMessage()),
+                e);
           }
         }
       }
-    } catch (IOException e) {
-      throw new IOException(
-          String.format(
-              "Failed to fetch a content from URL %s. Make sure the URL"
-                  + " is correct. Additionally, if you're using "
-                  + " relative form, make sure the referenced files are actually stored"
-                  + " relative to the devfile on the same host,"
-                  + " or try to specify URL in absolute form. The current attempt to download"
-                  + " the file failed with the following error message: %s",
-              fileURL, e.getMessage()),
-          e);
     } catch (ScmConfigurationPersistenceException
         | UnsatisfiedScmPreconditionException
         | ScmUnauthorizedException
@@ -109,4 +109,8 @@ public class AuthorizingFileContentProvider<T extends RemoteFactoryUrl>
       throw new DevfileException(e.getMessage(), e);
     }
   }
+
+  protected String formatAuthorization(String token) {
+    return "Bearer " + token;
+  }
 }
